@@ -10,9 +10,11 @@ import re
 from dataclasses import asdict
 
 from pymongo.collection import ReturnDocument
+from packaging import version
 from alfred3.alfredlog import QueuedLoggingInterface
 from alfred3.experiment import ExperimentSession
 from alfred3.exceptions import SessionTimeout
+from alfred3 import __version__ as alfred_version
 
 from .group import Group
 from .group import GroupManager
@@ -24,6 +26,8 @@ from ._util import MatchingError
 from ._util import BusyGroup
 from ._util import NoMatch
 
+ALFRED_VERSION = version.parse(alfred_version)
+
 class MatchMakerIO:
     def __init__(self, matchmaker):
         self.mm = matchmaker
@@ -248,8 +252,6 @@ class MatchMaker:
                     txt = f"You have successfully matched to role: {group.me.role}"
                     self += al.Text(txt, align="center")
 
-
-
     """
 
     _TIMEOUT_MSG = "MatchMaking timeout"
@@ -538,6 +540,30 @@ class MatchMaker:
         self._save_infos()
         return self.group
     
+
+    def check_activation(self):
+        """
+        Verifies that the MatchMaker is active and aborts the experiment
+        if necessary. 
+        
+        If the MatchMaker not active, the current experiment session is 
+        aborted, and the *abort_page* specified on initialization of 
+        the MatchMaker will be shown to participants.
+
+        The matching methods :meth:`.match_groupwise` and 
+        :meth:`.match_stepwise` will check for activation themselves,
+        when called. But this may be at a point in the experiment where
+        participants have made considerable progress. So it may be
+        useful to manually check for activation at an earlier stage.
+
+        As a general rule of thumb, it may be sensible to check activation
+        on first hiding of the landing page of an experiment. This way,
+        participants still get a nice welcoming screen, but do not invest
+        too much time before the experiment is cancelled.
+
+        """
+        self._check_activation()
+
     def _init_member(self):
         if self.member:
             return self.member
